@@ -65,7 +65,8 @@ type file struct {
 	referenceableSymbols map[ir.FullName]*symbol
 	referenceSymbols     []*symbol
 	symbols              []*symbol
-	diagnostics          []protocol.Diagnostic
+	irReport             *report.Report        // IR diagnostic report for code actions
+	diagnostics          []protocol.Diagnostic // Converted LSP diagnostics
 	cancelChecks         func()
 }
 
@@ -228,6 +229,7 @@ func (f *file) RefreshIR(ctx context.Context) {
 	)
 	f.diagnostics = nil
 	f.ir = nil
+	f.irReport = nil
 
 	// Opener creates a cached view of all files in the workspace.
 	pathToFiles := f.workspace.PathToFile()
@@ -281,6 +283,9 @@ func (f *file) RefreshIR(ctx context.Context) {
 			file.IndexSymbols(ctx)
 		}
 	}
+	// Store the IR report for code actions
+	f.irReport = diagnosticReport
+
 	// Only hold on to diagnostics where the primary span is for this path.
 	fileDiagnostics := xslices.Filter(diagnosticReport.Diagnostics, func(d report.Diagnostic) bool {
 		return d.Primary().Path() == f.objectInfo.Path()
@@ -562,12 +567,16 @@ func (f *file) irToSymbols(irSymbol ir.Symbol) ([]*symbol, []*symbol) {
 		resolved = append(resolved, tag)
 		unresolved = append(unresolved, f.messageToSymbols(irSymbol.AsMember().Options())...)
 	case ir.SymbolKindField:
+		// Get the type span. We prefer AsPath() which excludes modifiers like "repeated"
+		// while keeping package qualifiers, but fallback to RemovePrefixes() if AsPath is zero.
+		typeSpan := irSymbol.AsMember().TypeAST().AsPath().Span()
+		if typeSpan.IsZero() {
+			typeSpan = irSymbol.AsMember().TypeAST().RemovePrefixes().Span()
+		}
 		typ := &symbol{
 			ir:   irSymbol,
 			file: f,
-			// We remove prefixes from the type, since we want to exclude the prefix from the
-			// symbol, e.g. repeated string values = 1;, we want to capture the symbol for "string".
-			span: irSymbol.AsMember().TypeAST().RemovePrefixes().Span(),
+			span: typeSpan,
 		}
 		kind, needsResolution := getKindForMember(irSymbol.AsMember())
 		typ.kind = kind
@@ -598,7 +607,25 @@ func (f *file) irToSymbols(irSymbol ir.Symbol) ([]*symbol, []*symbol) {
 		resolved = append(resolved, tag)
 		unresolved = append(unresolved, f.messageToSymbols(irSymbol.AsMember().Options())...)
 	case ir.SymbolKindExtension:
-		// The symbol only includes the extension field name.
+		// Track the type reference for the extension field (e.g., "Testing" in "Testing testing = 89181;")
+		typeSpan := irSymbol.AsMember().TypeAST().AsPath().Span()
+		if typeSpan.IsZero() {
+			typeSpan = irSymbol.AsMember().TypeAST().RemovePrefixes().Span()
+		}
+		typ := &symbol{
+			ir:   irSymbol,
+			file: f,
+			span: typeSpan,
+		}
+		kind, needsResolution := getKindForMember(irSymbol.AsMember())
+		typ.kind = kind
+		if needsResolution {
+			unresolved = append(unresolved, typ)
+		} else {
+			resolved = append(resolved, typ)
+		}
+
+		// Track the extension field name itself.
 		ext := &symbol{
 			ir:   irSymbol,
 			file: f,
@@ -649,10 +676,15 @@ func (f *file) irToSymbols(irSymbol ir.Symbol) ([]*symbol, []*symbol) {
 		// Method input must be a single message type.
 		if astInputs := irSymbol.AsMethod().AST().AsMethod().Signature.Inputs(); astInputs.Len() == 1 {
 			inputAST := astInputs.At(0)
+			// Get the type span, preferring AsPath() but falling back to RemovePrefixes().
+			inputSpan := inputAST.AsPath().Span()
+			if inputSpan.IsZero() {
+				inputSpan = inputAST.RemovePrefixes().Span()
+			}
 			inputSym := &symbol{
 				ir:   irSymbol,
 				file: f,
-				span: inputAST.RemovePrefixes().Span(), // We always strip prefixes in case of streaming.
+				span: inputSpan,
 				kind: &reference{
 					def:      input.AST(), // Only messages can be method inputs and outputs
 					fullName: input.FullName(),
@@ -664,10 +696,15 @@ func (f *file) irToSymbols(irSymbol ir.Symbol) ([]*symbol, []*symbol) {
 		// Method output must be a single message type.
 		if astOutputs := irSymbol.AsMethod().AST().AsMethod().Signature.Outputs(); astOutputs.Len() == 1 {
 			outputAST := astOutputs.At(0)
+			// Get the type span, preferring AsPath() but falling back to RemovePrefixes().
+			outputSpan := outputAST.AsPath().Span()
+			if outputSpan.IsZero() {
+				outputSpan = outputAST.RemovePrefixes().Span()
+			}
 			outputSym := &symbol{
 				ir:   irSymbol,
 				file: f,
-				span: outputAST.RemovePrefixes().Span(), // We always strip prefixes in case of streaming.
+				span: outputSpan,
 				kind: &reference{
 					def:      output.AST(), // Only messages can be method inputs and outputs
 					fullName: output.FullName(),
@@ -769,6 +806,7 @@ func (f *file) messageToSymbolsHelper(msg ir.MessageValue, index int, parents []
 				// Otherwise, we get the component for the corresponding index.
 				span = components[index].Span()
 			}
+			span = trimLeadingDot(span)
 			sym := &symbol{
 				// NOTE: no [ir.Symbol] for option elements
 				file: f,
@@ -795,9 +833,10 @@ func (f *file) messageToSymbolsHelper(msg ir.MessageValue, index int, parents []
 					if component.IsLast() {
 						continue
 					}
+					componentSpan := trimLeadingDot(component.Span())
 					found := false
 					for _, parent := range parents {
-						if parent.span == component.Span() {
+						if parent.span == componentSpan {
 							found = true
 							break
 						}
@@ -806,7 +845,7 @@ func (f *file) messageToSymbolsHelper(msg ir.MessageValue, index int, parents []
 						sym := &symbol{
 							// NOTE: no [ir.Symbol] for option elements
 							file: f,
-							span: component.Span(),
+							span: componentSpan,
 							kind: &option{
 								def:             parentType.AsValue().Field().AST(),
 								defFullName:     parentType.AsValue().Field().FullName(),
@@ -1069,3 +1108,21 @@ func (f *file) GetSymbols(query string) iter.Seq[protocol.SymbolInformation] {
 		}
 	}
 }
+
+// trimLeadingDot removes the leading dot from a span if present.
+//
+// Option field references in protobuf use dot notation like "(extension).field",
+// but the AST component span includes the dot separator (e.g., ".field" instead
+// of "field"). We need to trim this to get the correct symbol span for renaming,
+// otherwise we'd end up renaming ".field" to ".newname", resulting in invalid
+// syntax like "(extension)newname" instead of "(extension).newname".
+func trimLeadingDot(span source.Span) source.Span {
+	if span.Text() != "" && span.Text()[0] == '.' {
+		return source.Span{
+			File:  span.File,
+			Start: span.Start + 1,
+			End:   span.End,
+		}
+	}
+	return span
+}
