@@ -18,16 +18,19 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"regexp"
 	"runtime/debug"
 	"strings"
 	"unicode/utf16"
 
+	celpv "buf.build/go/protovalidate/cel"
 	"buf.build/go/standard/xslices"
 	"github.com/bufbuild/buf/private/buf/bufformat"
-	"github.com/bufbuild/protocompile/experimental/ir"
 	"github.com/bufbuild/protocompile/parser"
 	"github.com/bufbuild/protocompile/reporter"
+	"github.com/google/cel-go/cel"
 	"go.lsp.dev/protocol"
+	"mvdan.cc/xurls/v2"
 )
 
 const (
@@ -36,49 +39,6 @@ const (
 	maxSymbolResults = 1000
 )
 
-// The subset of SemanticTokenTypes that we support.
-// Must match the order of [semanticTypeLegend].
-// https://microsoft.github.io/language-server-protocol/specifications/lsp/3.18/specification/#textDocument_semanticTokens
-const (
-	semanticTypeType = iota
-	semanticTypeStruct
-	semanticTypeVariable
-	semanticTypeEnum
-	semanticTypeEnumMember
-	semanticTypeInterface
-	semanticTypeMethod
-	semanticTypeDecorator
-	semanticTypeNamespace
-)
-
-// The subset of SemanticTokenModifiers that we support.
-// Must match the order of [semanticModifierLegend].
-// Semantic modifiers are encoded as a bitset, hence the shifted iota.
-// https://microsoft.github.io/language-server-protocol/specifications/lsp/3.18/specification/#textDocument_semanticTokens
-const (
-	semanticModifierDeprecated = 1 << iota
-	semanticModifierDefaultLibrary
-)
-
-var (
-	// These slices must match the order of the indices in the above const blocks.
-	semanticTypeLegend = []string{
-		"type",
-		"struct",
-		"variable",
-		"enum",
-		"enumMember",
-		"interface",
-		"method",
-		"decorator",
-		"namespace",
-	}
-	semanticModifierLegend = []string{
-		"deprecated",
-		"defaultLibrary", // maps to builtin values
-	}
-)
-
 // server is an implementation of [protocol.Server].
 //
 // This is a separate type from buflsp.lsp so that the dozens of handler methods for this
@@ -92,11 +52,31 @@ type server struct {
 
 	// We embed the LSP pointer as well, since it only has private members.
 	*lsp
+
+	// httpsURLRegex is used to find https:// URLs in comments for document links.
+	httpsURLRegex *regexp.Regexp
+	// celEnv is the CEL environment used for parsing protovalidate expressions.
+	celEnv *cel.Env
 }
 
 // newServer creates a protocol.Server implementation out of an lsp.
-func newServer(lsp *lsp) protocol.Server {
-	return &server{lsp: lsp}
+func newServer(lsp *lsp) (protocol.Server, error) {
+	httpsURLRegex, err := xurls.StrictMatchingScheme("https://")
+	if err != nil {
+		return nil, fmt.Errorf("failed to create HTTPS URL regex: %w", err)
+	}
+	celEnv, err := cel.NewEnv(
+		cel.Lib(celpv.NewLibrary()),
+		cel.EnableMacroCallTracking(),
+	)
+	if err != nil {
+		return nil, fmt.Errorf("failed to create CEL environment: %w", err)
+	}
+	return &server{
+		lsp:           lsp,
+		httpsURLRegex: httpsURLRegex,
+		celEnv:        celEnv,
+	}, nil
 }
 
 // Methods for server are grouped according to the groups in the LSP protocol specification.
@@ -143,6 +123,11 @@ func (s *server) Initialize(
 					IncludeText: false,
 				},
 			},
+			CodeActionProvider: &protocol.CodeActionOptions{
+				CodeActionKinds: []protocol.CodeActionKind{
+					protocol.SourceOrganizeImports,
+				},
+			},
 			CompletionProvider: &protocol.CompletionOptions{
 				ResolveProvider:   true,
 				TriggerCharacters: []string{".", "\"", "/"},
@@ -150,6 +135,7 @@ func (s *server) Initialize(
 			DefinitionProvider:         &protocol.DefinitionOptions{},
 			TypeDefinitionProvider:     &protocol.TypeDefinitionOptions{},
 			DocumentFormattingProvider: true,
+			DocumentHighlightProvider:  true,
 			HoverProvider:              true,
 			ReferencesProvider:         &protocol.ReferenceOptions{},
 			RenameProvider: &protocol.RenameOptions{
@@ -164,6 +150,8 @@ func (s *server) Initialize(
 			},
 			WorkspaceSymbolProvider: true,
 			DocumentSymbolProvider:  true,
+			FoldingRangeProvider:    true,
+			DocumentLinkProvider:    &protocol.DocumentLinkOptions{},
 		},
 		ServerInfo: info,
 	}, nil
@@ -439,90 +427,7 @@ func (s *server) SemanticTokensFull(
 	ctx context.Context,
 	params *protocol.SemanticTokensParams,
 ) (*protocol.SemanticTokens, error) {
-	file := s.fileManager.Get(params.TextDocument.URI)
-	if file == nil {
-		return nil, nil
-	}
-	// In the case where there are no symbols for the file, we return nil for SemanticTokensFull.
-	// This is based on the specification for the method textDocument/semanticTokens/full,
-	// the expected response is the union type `SemanticTokens | null`.
-	// https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#textDocument_semanticTokens
-	if len(file.symbols) == 0 {
-		return nil, nil
-	}
-	// This fairly painful encoding is described in detail here:
-	// https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#textDocument_semanticTokens
-	var (
-		encoded           []uint32
-		prevLine, prevCol uint32
-	)
-	for _, symbol := range file.symbols {
-		var semanticType uint32
-		var semanticModifier uint32
-		_, ok := symbol.kind.(*option)
-		if ok {
-			semanticType = semanticTypeDecorator
-		} else {
-			switch symbol.ir.Kind() {
-			case ir.SymbolKindPackage:
-				semanticType = semanticTypeNamespace
-			case ir.SymbolKindMessage:
-				semanticType = semanticTypeStruct
-			case ir.SymbolKindEnum:
-				semanticType = semanticTypeEnum
-			case ir.SymbolKindField:
-				// For predeclared types, we set semanticType to semanticTypeType
-				if symbol.IsBuiltIn() {
-					semanticType = semanticTypeType
-					semanticModifier += semanticModifierDefaultLibrary
-				} else {
-					semanticType = semanticTypeVariable
-				}
-			case ir.SymbolKindEnumValue:
-				semanticType = semanticTypeEnumMember
-			case ir.SymbolKindExtension:
-				semanticType = semanticTypeVariable
-			case ir.SymbolKindService:
-				semanticType = semanticTypeInterface
-			case ir.SymbolKindMethod:
-				semanticType = semanticTypeMethod
-			default:
-				continue
-			}
-		}
-		if _, ok := symbol.ir.Deprecated().AsBool(); ok {
-			semanticModifier += semanticModifierDeprecated
-		}
-
-		startLocation := symbol.span.Location(symbol.span.Start, positionalEncoding)
-		endLocation := symbol.span.Location(symbol.span.End, positionalEncoding)
-
-		for i := startLocation.Line; i <= endLocation.Line; i++ {
-			newLine := uint32(i - 1)
-			var newCol uint32
-			if i == startLocation.Line {
-				newCol = uint32(startLocation.Column - 1)
-				if prevLine == newLine {
-					newCol -= prevCol
-				}
-			}
-			symbolLen := uint32(endLocation.Column - 1)
-			if i == startLocation.Line {
-				symbolLen -= uint32(startLocation.Column - 1)
-			}
-			encoded = append(encoded, newLine-prevLine, newCol, symbolLen, semanticType, semanticModifier)
-			prevLine = newLine
-			if i == startLocation.Line {
-				prevCol = uint32(startLocation.Column - 1)
-			} else {
-				prevCol = 0
-			}
-		}
-	}
-	if len(encoded) == 0 {
-		return nil, nil
-	}
-	return &protocol.SemanticTokens{Data: encoded}, nil
+	return semanticTokensFull(s.fileManager.Get(params.TextDocument.URI), s.celEnv)
 }
 
 // Symbols is the entry point for workspace-wide symbol search.
@@ -562,6 +467,36 @@ func (s *server) DocumentSymbol(ctx context.Context, params *protocol.DocumentSy
 	return anyResults, nil
 }
 
+// DocumentHighlight is the entry point for highlighting occurrences of a symbol within a file.
+//
+// Supported symbol types for highlighting are [referenceable] and [reference] (messages, enums,
+// extensions, and their references). All highlights use Text kind.
+// Services, RPC methods, enum values, and field names are not highlighted.
+func (s *server) DocumentHighlight(ctx context.Context, params *protocol.DocumentHighlightParams) ([]protocol.DocumentHighlight, error) {
+	symbol := s.getSymbol(ctx, params.TextDocument.URI, params.Position)
+	if symbol == nil {
+		return nil, nil
+	}
+	return symbol.DocumentHighlights(), nil
+}
+
+// CodeAction is called when the client requests code actions for a given range.
+func (s *server) CodeAction(ctx context.Context, params *protocol.CodeActionParams) ([]protocol.CodeAction, error) {
+	file := s.fileManager.Get(params.TextDocument.URI)
+	if file == nil {
+		return nil, nil
+	}
+	codeActionSet := xslices.ToStructMap(params.Context.Only)
+
+	var actions []protocol.CodeAction
+	if _, ok := codeActionSet[protocol.SourceOrganizeImports]; len(codeActionSet) == 0 || ok {
+		if organizeImportsAction := s.getOrganizeImportsCodeAction(ctx, file); organizeImportsAction != nil {
+			actions = append(actions, *organizeImportsAction)
+		}
+	}
+	return actions, nil
+}
+
 // PrepareRename is the entry point for checking workspace wide renaming of a symbol.
 //
 // If a symbol can be renamed, PrepareRename will return the range for the rename. Returning
@@ -576,6 +511,15 @@ func (s *server) PrepareRename(ctx context.Context, params *protocol.PrepareRena
 	}
 	switch symbol.kind.(type) {
 	case *referenceable, *static, *reference:
+		// Don't allow renaming symbols defined in non-local files (e.g., WKTs from ~/.cache)
+		// Check the definition if available, otherwise check the symbol's own file
+		defFile := symbol.file
+		if symbol.def != nil && symbol.def.file != nil {
+			defFile = symbol.def.file
+		}
+		if defFile != nil && !defFile.IsLocal() {
+			return nil, fmt.Errorf("cannot rename a symbol in a non-local file")
+		}
 		rnge := reportSpanToProtocolRange(symbol.span)
 		return &rnge, nil
 	}
@@ -591,9 +535,42 @@ func (s *server) Rename(
 	if symbol == nil {
 		return nil, nil
 	}
+	// Don't allow renaming symbols defined in non-local files (e.g., WKTs from ~/.cache)
+	// Check the definition if available, otherwise check the symbol's own file
+	defFile := symbol.file
+	if symbol.def != nil && symbol.def.file != nil {
+		defFile = symbol.def.file
+	}
+	if defFile != nil && !defFile.IsLocal() {
+		return nil, fmt.Errorf("cannot rename a symbol in a non-local file")
+	}
 	return symbol.Rename(params.NewName)
 }
 
+// FoldingRanges is the entry point for folding ranges, which allows collapsing/expanding blocks of code.
+func (s *server) FoldingRanges(
+	ctx context.Context,
+	params *protocol.FoldingRangeParams,
+) ([]protocol.FoldingRange, error) {
+	file := s.fileManager.Get(params.TextDocument.URI)
+	if file == nil {
+		return nil, nil
+	}
+	return s.foldingRange(file), nil
+}
+
+// DocumentLink is the entry point for document links, which makes imports and URLs clickable.
+func (s *server) DocumentLink(
+	ctx context.Context,
+	params *protocol.DocumentLinkParams,
+) ([]protocol.DocumentLink, error) {
+	file := s.fileManager.Get(params.TextDocument.URI)
+	if file == nil {
+		return nil, nil
+	}
+	return s.documentLink(file), nil
+}
+
 // getSymbol is a helper function that gets the *[symbol] for the given [protocol.URI] and
 // [protocol.Position].
 func (s *server) getSymbol(
